@@ -11,8 +11,8 @@
  * any later version.
  *
  * You should have received a copy of the GNU General Public License
- * (for example COPYING); if not, write to the Free
- * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * (for example COPYING); if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #include <stdbool.h>
@@ -44,7 +44,7 @@
 #include "os_darwin.h"
 
 // Needed by '-V' option (CVS versioning) of smartd/smartctl
-const char *os_XXXX_c_cvsid="$Id: os_darwin.cpp,v 1.20 2008/03/04 22:09:47 ballen4705 Exp $" \
+const char *os_XXXX_c_cvsid="$Id: os_darwin.cpp 3728 2012-12-13 17:57:50Z chrfranke $" \
 ATACMDS_H_CVSID CONFIG_H_CVSID INT64_H_CVSID OS_DARWIN_H_CVSID SCSICMDS_H_CVSID UTILITY_H_CVSID;
 
 // Print examples for smartctl.
@@ -435,6 +435,21 @@ int escalade_command_interface(int fd, int escalade_port, int escalade_type,
   return -1;
 }
 
+int areca_command_interface(int fd, int escalade_port, 
+			       smart_command_set command, int select,
+			       char *data)
+{
+  fd = fd;
+  escalade_port = escalade_port;
+  command = command;
+  select = select;
+  data = data;
+  return -1;
+}
+
+
+
+
 int marvell_command_interface(int fd, smart_command_set command,
 		      int select, char *data)
 { 
