@@ -40,10 +40,6 @@
 #include <sys/param.h>
 #endif
 
-#if defined(__QNXNTO__) 
-#include <new> // TODO: Why is this include necessary on QNX ?
-#endif
-
 #include "int64.h"
 #include "atacmds.h"
 #include "dev_interface.h"
@@ -54,7 +50,7 @@
 #include "smartctl.h"
 #include "utility.h"
 
-const char * smartctl_cpp_cvsid = "$Id: smartctl.cpp 3634 2012-10-09 11:45:55Z chrfranke $"
+const char * smartctl_cpp_cvsid = "$Id: smartctl.cpp 3726 2012-12-12 20:02:48Z chrfranke $"
   CONFIG_H_CVSID SMARTCTL_H_CVSID;
 
 // Globals to control printing
@@ -89,7 +85,7 @@ static void Usage()
 "  --identify[=[w][nvb]]\n"
 "         Show words and bits from IDENTIFY DEVICE data                (ATA)\n\n"
 "  -g NAME, --get=NAME\n"
-"        Get device setting: all, aam, apm, lookahead, security, wcache\n\n"
+"        Get device setting: all, aam, apm, lookahead, security, wcache, rcache\n\n"
 "  -a, --all\n"
 "         Show all SMART information for device\n\n"
 "  -x, --xall\n"
@@ -125,7 +121,7 @@ static void Usage()
 "  -s NAME[,VALUE], --set=NAME[,VALUE]\n"
 "        Enable/disable/change device setting: aam,[N|off], apm,[N|off],\n"
 "        lookahead,[on|off], security-freeze, standby,[N|off|now],\n"
-"        wcache,[on|off]\n\n"
+"        wcache,[on|off], rcache,[on|off]\n\n"
   );
   printf(
 "======================================= READ AND DISPLAY DATA OPTIONS =====\n\n"
@@ -222,10 +218,10 @@ static std::string getvalidarglist(int opt)
   case 'f':
     return "old, brief, hex[,id|val]";
   case 'g':
-    return "aam, apm, lookahead, security, wcache";
+    return "aam, apm, lookahead, security, wcache, rcache";
   case opt_set:
     return "aam,[N|off], apm,[N|off], lookahead,[on|off], security-freeze, "
-           "standby,[N|off|now], wcache,[on|off]";
+           "standby,[N|off|now], wcache,[on|off], rcache,[on|off]";
   case 's':
     return getvalidarglist(opt_smart)+", "+getvalidarglist(opt_set);
   case opt_identify:
@@ -494,7 +490,7 @@ static const char * parse_options(int argc, char** argv,
         unsigned interval = 0; int n1 = -1, n2 = -1, len = strlen(optarg);
         if (!(   sscanf(optarg,"scttempint,%u%n,p%n", &interval, &n1, &n2) == 1
               && 0 < interval && interval <= 0xffff && (n1 == len || n2 == len))) {
-            strcpy(extraerror, "Option -l scttempint,N[,p] must have positive integer N\n");
+            snprintf(extraerror, sizeof(extraerror), "Option -l scttempint,N[,p] must have positive integer N\n");
             badarg = true;
         }
         ataopts.sct_temp_int = interval;
@@ -550,7 +546,7 @@ static const char * parse_options(int argc, char** argv,
           ataopts.sct_erc_writetime = wt;
         }
         else {
-          sprintf(extraerror, "Option -l scterc,[READTIME,WRITETIME] syntax error\n");
+          snprintf(extraerror, sizeof(extraerror), "Option -l scterc,[READTIME,WRITETIME] syntax error\n");
           badarg = true;
         }
       } else if (   !strncmp(optarg, "gplog,"   , sizeof("gplog,"   )-1)
@@ -566,14 +562,14 @@ static const char * parse_options(int argc, char** argv,
         const char * erropt = (gpl ? "gplog" : "smartlog");
         if (!(   n1 == len || n2 == len
               || (n3 == len && (sign == '+' || sign == '-')))) {
-          sprintf(extraerror, "Option -l %s,ADDR[,FIRST[-LAST|+SIZE]] syntax error\n", erropt);
+          snprintf(extraerror, sizeof(extraerror), "Option -l %s,ADDR[,FIRST[-LAST|+SIZE]] syntax error\n", erropt);
           badarg = true;
         }
         else if (!(    logaddr <= 0xff && page <= (gpl ? 0xffffU : 0x00ffU)
                    && 0 < nsectors
                    && (nsectors <= (gpl ? 0xffffU : 0xffU) || nsectors == ~0U)
                    && (sign != '-' || page <= nsectors)                       )) {
-          sprintf(extraerror, "Option -l %s,ADDR[,FIRST[-LAST|+SIZE]] parameter out of range\n", erropt);
+          snprintf(extraerror, sizeof(extraerror), "Option -l %s,ADDR[,FIRST[-LAST|+SIZE]] parameter out of range\n", erropt);
           badarg = true;
         }
         else {
@@ -636,6 +632,7 @@ static const char * parse_options(int argc, char** argv,
       ataopts.get_aam = ataopts.get_apm = true;
       ataopts.get_security = true;
       ataopts.get_lookahead = ataopts.get_wcache = true;
+      scsiopts.get_rcd = scsiopts.get_wce = true;
       scsiopts.smart_background_log = true;
       scsiopts.smart_ss_media_log = true;
       scsiopts.sasphy = true;
@@ -693,6 +690,7 @@ static const char * parse_options(int argc, char** argv,
         ataopts.smart_selftest_type = CONVEYANCE_SELF_TEST;
       } else if (!strcmp(optarg,"force")) {
         ataopts.smart_selftest_force = true;
+        scsiopts.smart_selftest_force = true;
       } else if (!strcmp(optarg,"afterselect,on")) {
         // scan remainder of disk after doing selected segment
         ataopts.smart_selective_args.scan_after_select = 2;
@@ -706,10 +704,10 @@ static const char * parse_options(int argc, char** argv,
 	errno=0;
 	i=(int)strtol(optarg+strlen("pending,"), &tailptr, 10);
 	if (errno || *tailptr != '\0') {
-	  sprintf(extraerror, "Option -t pending,N requires N to be a non-negative integer\n");
+          snprintf(extraerror, sizeof(extraerror), "Option -t pending,N requires N to be a non-negative integer\n");
           badarg = true;
 	} else if (i<0 || i>65535) {
-	  sprintf(extraerror, "Option -t pending,N (N=%d) must have 0 <= N <= 65535\n", i);
+          snprintf(extraerror, sizeof(extraerror), "Option -t pending,N (N=%d) must have 0 <= N <= 65535\n", i);
           badarg = true;
 	} else {
           ataopts.smart_selective_args.pending_time = i+1;
@@ -720,15 +718,15 @@ static const char * parse_options(int argc, char** argv,
         // parse range of LBAs to test
         uint64_t start, stop; int mode;
         if (split_selective_arg(optarg, &start, &stop, &mode)) {
-	  sprintf(extraerror, "Option -t select,M-N must have non-negative integer M and N\n");
+          snprintf(extraerror, sizeof(extraerror), "Option -t select,M-N must have non-negative integer M and N\n");
           badarg = true;
         } else {
           if (ataopts.smart_selective_args.num_spans >= 5 || start > stop) {
             if (start > stop) {
-              sprintf(extraerror, "ERROR: Start LBA (%"PRIu64") > ending LBA (%"PRId64") in argument \"%s\"\n",
+              snprintf(extraerror, sizeof(extraerror), "ERROR: Start LBA (%"PRIu64") > ending LBA (%"PRId64") in argument \"%s\"\n",
                 start, stop, optarg);
             } else {
-              sprintf(extraerror,"ERROR: No more than five selective self-test spans may be"
+              snprintf(extraerror, sizeof(extraerror),"ERROR: No more than five selective self-test spans may be"
                 " defined\n");
             }
             badarg = true;
@@ -740,13 +738,13 @@ static const char * parse_options(int argc, char** argv,
           ataopts.smart_selftest_type = SELECTIVE_SELF_TEST;
         }
       } else if (!strncmp(optarg, "scttempint", sizeof("scstempint")-1)) {
-        strcpy(extraerror, "-t scttempint is no longer supported, use -l scttempint instead\n");
+        snprintf(extraerror, sizeof(extraerror), "-t scttempint is no longer supported, use -l scttempint instead\n");
         badarg = true;
       } else if (!strncmp(optarg, "vendor,", sizeof("vendor,")-1)) {
         unsigned subcmd = ~0U; int n = -1;
         if (!(   sscanf(optarg, "%*[a-z],0x%x%n", &subcmd, &n) == 1
               && subcmd <= 0xff && n == (int)strlen(optarg))) {
-          strcpy(extraerror, "Option -t vendor,0xNN syntax error\n");
+          snprintf(extraerror, sizeof(extraerror), "Option -t vendor,0xNN syntax error\n");
           badarg = true;
         }
         else
@@ -832,6 +830,7 @@ static const char * parse_options(int argc, char** argv,
             ataopts.get_aam = ataopts.get_apm = true;
             ataopts.get_security = true;
             ataopts.get_lookahead = ataopts.get_wcache = true;
+            scsiopts.get_rcd = scsiopts.get_wce = true;
           }
           else if (!strcmp(name, "aam")) {
             if (get)
@@ -841,7 +840,7 @@ static const char * parse_options(int argc, char** argv,
             else if (val <= 254)
               ataopts.set_aam = val + 1;
             else {
-              sprintf(extraerror, "Option -s aam,N must have 0 <= N <= 254\n");
+              snprintf(extraerror, sizeof(extraerror), "Option -s aam,N must have 0 <= N <= 254\n");
               badarg = true;
             }
           }
@@ -853,13 +852,14 @@ static const char * parse_options(int argc, char** argv,
             else if (1 <= val && val <= 254)
               ataopts.set_apm = val + 1;
             else {
-              sprintf(extraerror, "Option -s apm,N must have 1 <= N <= 254\n");
+              snprintf(extraerror, sizeof(extraerror), "Option -s apm,N must have 1 <= N <= 254\n");
               badarg = true;
             }
           }
           else if (!strcmp(name, "lookahead")) {
-            if (get)
+            if (get) {
               ataopts.get_lookahead = true;
+            }
             else if (off)
               ataopts.set_lookahead = -1;
             else if (on)
@@ -867,6 +867,16 @@ static const char * parse_options(int argc, char** argv,
             else
               badarg = true;
           }
+          else if (!strcmp(name, "rcache")) {
+            if (get)
+              scsiopts.get_rcd = true;
+            else if (off)
+              scsiopts.set_rcd = -1;
+            else if (on)
+              scsiopts.set_rcd = 1;
+            else
+              badarg = true;
+          }
           else if (get && !strcmp(name, "security")) {
             ataopts.get_security = true;
           }
@@ -882,17 +892,23 @@ static const char * parse_options(int argc, char** argv,
             else if (val <= 255)
               ataopts.set_standby = val + 1;
             else {
-              sprintf(extraerror, "Option -s standby,N must have 0 <= N <= 255\n");
+              snprintf(extraerror, sizeof(extraerror), "Option -s standby,N must have 0 <= N <= 255\n");
               badarg = true;
             }
           }
           else if (!strcmp(name, "wcache")) {
-            if (get)
+            if (get) {
               ataopts.get_wcache = true;
-            else if (off)
+              scsiopts.get_wce = true;
+            }
+            else if (off) {
               ataopts.set_wcache = -1;
-            else if (on)
+              scsiopts.set_wce = -1;
+            }
+            else if (on) {
               ataopts.set_wcache = 1;
+              scsiopts.set_wce = 1;
+            }
             else
               badarg = true;
           }
